#!/bin/bash

set -x

enable_start_service()
{
    args=("$@")
    i="0"
    while [[  ! -z "${args[i]// }"  ]]; do
        systemctl enable ${args[i]// }
        systemctl start ${args[i]// }
        i=$[$i + 1]
    done
}

package()
{
    args=("$@")
    default="install"
    i="0"
    while [[  ! -z "${args[i]// }"  ]]; do
        yum $default -y ${args[i]// }
        i=$[$i + 1]
    done
}

# Disable SELINUX (for now)
sed -i 's/SELINUX=enforcing/SELINUX=disabled/' /etc/selinux/config
setenforce 0

package epel-release
package wget postgresql-server postgresql-contrib  nginx redis

#### Redis configuration
echo 'unixsocket /var/run/redis/redis.sock' | sudo tee -a /etc/redis.conf
echo 'unixsocketperm 770' | sudo tee -a /etc/redis.conf
mkdir /var/run/redis
chown redis:redis /var/run/redis
chmod 755 /var/run/redis
if [ -d /etc/tmpfiles.d ]; then
  echo 'd  /var/run/redis  0755  redis  redis  10d  -' | sudo tee -a /etc/tmpfiles.d/redis.conf
fi

### Postgresql config
postgresql-setup initdb
cat << EOF > /var/lib/pgsql/data/pg_hba.conf
# METHOD can be "trust", "reject", "md5", "password", "gss", "sspi",

# TYPE  DATABASE        USER            ADDRESS                 METHOD
local   all             all                                     trust
host    all             all             127.0.0.1/32            trust
host    all             all             ::1/128                 trust
EOF
enable_start_service postgresql redis
sudo usermod -aG redis git

sudo -u postgres createuser git -d -l
psql -U git -d template1 -c "create database gitlabhq_production with owner git"

wget -P /etc/yum.repos.d/ https://copr.fedorainfracloud.org/coprs/softwarepublico/v4/repo/epel-7/softwarepublico-v4-epel-7.repo
package gitlab
cd /usr/lib/gitlab
sed -i 's,http://localhost:8080/,http://localhost:8080/gitlab,' /etc/gitlab-shell/config.yml

echo 'production: redis://localhost:6379' > /usr/lib/gitlab/config/resque.yml
sed -i 's,localhost,localhost\n    relative_url_root: /gitlab,' /etc/gitlab/gitlab.yml

cat << EOF > /usr/lib/gitlab/config/initializers/gitlab_shell_secret_token.rb
Gitlab::Application.configure do
 config.relative_url_root = "/gitlab"
end
EOF
sed -i 's/# ENV/ENV/' /etc/gitlab/unicorn.rb
sed -i 's/# config.relative_url_root/config.relative_url_root/' /usr/lib/gitlab/config/application.rb
chown -R git:git /usr/lib/gitlab/tmp/cache
chown -R git:git /usr/lib/gitlab/public/assets
chown -R git:git /var/lib/gitlab-assets
enable_start_service gitlab

sudo -u git bundle exec rake db:setup RAILS_ENV=production && touch /var/lib/gitlab/setup.done

cat << EOF > /etc/nginx/conf.d/gitlab.conf
upstream gitlab-workhorse {
   server unix:/usr/lib/gitlab/tmp/sockets/gitlab-workhorse.socket fail_timeout=10s;
}

server {
  listen *:8081 default_server;
  large_client_header_buffers 4 32k;
  server_name gitlab.dev; ## Replace this with something like gitlab.example.com
  server_tokens off; ## Don't show the nginx version number, a security best practice
  root /usr/lib/gitlab/public;

  client_max_body_size 20m;

  access_log  /var/log/nginx/gitlab_access.log;
  error_log   /var/log/nginx/gitlab_error.log;

location / {
try_files \$uri \$uri/index.html \$uri.html @gitlab;
}

location @gitlab {
    client_max_body_size 0;
    gzip off;

    proxy_read_timeout      300;
    proxy_connect_timeout   300;
    proxy_redirect          off;

    proxy_http_version 1.1;

    proxy_set_header    Host               \$http_host;
    proxy_set_header    X-Real-IP          \$remote_addr;
    proxy_set_header    X-Forwarded-For    \$proxy_add_x_forwarded_for;
    proxy_set_header    X-Forwarded-Proto  \$scheme;

    proxy_pass http://gitlab-workhorse;
}

  location ~ ^/(assets)/ {
    root /usr/lib/gitlab/public;

    gzip_static on; # to serve pre-gzipped version
    expires max;
    add_header Cache-Control public;
  }

  error_page 502 /502.html;
}
EOF

enable_start_service nginx


cd /usr/lib/gitlab &&
sudo -u git bundle exec rake assets:precompile RAILS_ENV=production


##### restore backup
if [ -e /vagrant/*_backup.tar ]; then
    backup=$(echo /vagrant/*_backup.tar)
elif [ -e /home/vagrant/sync/*_backup.tar ]; then
    backup=$(echo /home/vagrant/sync/*_backup.tar)
fi

if [[ -e $backup ]]; then
    printf "####################################################"
    printf "Restaurando backup"
    /etc/init.d/gitlab stop

    printf "Movendo backup para diretório correto"
    chown git.git $backup
    ln $backup /var/lib/gitlab/backups/
    cd /usr/lib/gitlab
    sudo -u git bundle exec rake gitlab:backup:restore RAILS_ENV=production force=yes
else
    printf "Backup não encontrado"
fi

printf "Instalando gitlab 8.5"

wget -P /etc/yum.repos.d/ https://copr.fedorainfracloud.org/coprs/softwarepublico/experimental/repo/epel-7/softwarepublico-experimental-epel-7.repo

package centos-release-scl rh-ruby22
package gitlab
sudo chmod -R ug+rwX,o-rwx /var/lib/gitlab-shell/repositories/
sudo chmod -R ug-s /var/lib/gitlab-shell/repositories/
sudo find /var/lib/gitlab-shell/repositories/ -type d -print0 | sudo xargs -0 chmod g+s
sudo chmod -R 777 /var/log/gitlab-shell/

rm /usr/lib/gitlab-shell/config.yml

cat << EOF > /usr/lib/gitlab-shell/config.yml
user: git
gitlab_url: "http://127.0.0.1:8080/gitlab"

http_settings:
#  user: someone
#  password: somepass
#  ca_file: /etc/ssl/cert.pem
#  ca_path: /etc/pki/tls/certs
  self_signed_cert: false

repos_path: "/var/lib/gitlab-shell/repositories/"
auth_file: "/var/lib/gitlab-shell/.ssh/authorized_keys"

redis:
  bin: /usr/bin/redis-cli
  host: 127.0.0.1
  port: 6379
  # pass: redispass # Allows you to specify the password for Redis
  #database: 0
  #socket: /var/run/redis/redis.sock # Comment out this line if you want to use TCP
  #namespace: resque:gitlab

log_file: "/var/log/gitlab-shell/gitlab-shell.log"
log_level: INFO
audit_usernames: false
EOF

chown -R git.git /var/lib/gitlab-shell/

systemctl restart gitlab

sudo -u git env PATH=/opt/rh/rh-ruby22/root/usr/bin/:/opt/rh/nodejs010/root/usr/bin/:/sbin:/bin:/usr/sbin:/usr/bin:/usr/X11R6/bin LD_LIBRARY_PATH=/opt/rh/rh-ruby22/root/usr/lib64:/opt/rh/nodejs010/root/lib64/:/opt/rh/v8314/root/usr/lib64/ bundle exec rake db:migrate RAILS_ENV=production

sudo -u git env PATH=/opt/rh/rh-ruby22/root/usr/bin/:/opt/rh/nodejs010/root/usr/bin/:/sbin:/bin:/usr/sbin:/usr/bin:/usr/X11R6/bin LD_LIBRARY_PATH=/opt/rh/rh-ruby22/root/usr/lib64:/opt/rh/nodejs010/root/lib64/:/opt/rh/v8314/root/usr/lib64/ bundle exec rake assets:precompile RAILS_ENV=production

#create new root user
sed -i "s|username: 'root'|username: 'devroot'|" /usr/lib/gitlab/db/fixtures/production/001_admin.rb

sudo -u git env PATH=/opt/rh/rh-ruby22/root/usr/bin/:/opt/rh/nodejs010/root/usr/bin/:/sbin:/bin:/usr/sbin:/usr/bin:/usr/X11R6/bin LD_LIBRARY_PATH=/opt/rh/rh-ruby22/root/usr/lib64:/opt/rh/nodejs010/root/lib64/:/opt/rh/v8314/root/usr/lib64/ bundle exec rake db:seed_fu RAILS_ENV=production

sudo -u git env PATH=/opt/rh/rh-ruby22/root/usr/bin/:/opt/rh/nodejs010/root/usr/bin/:/sbin:/bin:/usr/sbin:/usr/bin:/usr/X11R6/bin LD_LIBRARY_PATH=/opt/rh/rh-ruby22/root/usr/lib64:/opt/rh/nodejs010/root/lib64/:/opt/rh/v8314/root/usr/lib64/ bundle exec rake gitlab:check RAILS_ENV=production

systemctl restart gitlab
